#!/usr/bin/env python

import sys
from deter import topdl

def annotate_degree(top):
    '''
    Add an attribute (node_degree) to each Computer in the topology (top) that
    lists its degree
    '''
    for e in top.elements:
	# Skip elements that are not computers
	if not isinstance(e, topdl.Computer): continue
	# degree is the number of interfaces
	deg = len(e.interface)
	# Save a string-valued attribute
	e.set_attribute('node_degree', '%s' % deg)
    return top

def add_operating_system(top):
    '''
    Add an OperatingSystem class to each computer in the topology (top).  If
    the node is a leaf (degree 1) make it Ubuntu Linux, otherwise make it
    FreeBSD.  annotate_degree() must have been called on the topology as this
    routine uses the node_degree attribute to make decisions.
    '''

    for e in top.elements:
	# Skip non-Computers
	if not isinstance(e, topdl.Computer): continue
	a = e.get_attribute('node_degree')
	# if there is a node_degree attribute, assign an OS
	if a:
	    # Covnert the string attribute into an integer
	    try:
		deg = int(a)
	    except ValueError, e:
		sys.exit('%s has a non-integer degree %s!?' % (e.name, a))
	    # Assign the os - includes a distribution for Linux
	    if deg == 1:
		e.os.append(topdl.OperatingSystem(name='Linux', 
		    distribution='Ubuntu'))
	    else:
		e.os.append(topdl.OperatingSystem(name='FreeBSD'))
    return top

def add_links(top):
    '''
    For each router - Computer with FreeBSD assigned as an OS - select one of
    its Linux children and make a connection to one of the other router's
    children.
    '''

    def has_os(e, os_name):
	'''
	Return true if any of e's os classes has the given os_name
	'''
	if e.os is None: return False
	else: return any([ o.name == os_name for o in e.os ])

    # At the end of this loop, leaves contains a list of leaves for each router
    # in the topology.
    leaves = [ ]
    for e in top.elements:
	if not isinstance(e, topdl.Computer): continue
	if not has_os(e, 'FreeBSD'): continue
	# This is a Computer running FreeBsd
	my_leaves = []
	# walk the interfaces
	for inf in e.interface:
	    # Walk the interface's substrates
	    for s in inf.subs:
		# Walk the substrate's interfaces looking for the other
		# elements - not this router.
		for sinf in s.interfaces:
		    ee = sinf.element
		    # all the Linux computers get attached to the leaves dict
		    if isinstance(ee, topdl.Computer) and ee != e:
			if has_os(ee, 'Linux'):
			    my_leaves.append(ee)
	# One router has no leaves.  Ignore it.
	if len(my_leaves) > 0:
	    leaves.append(my_leaves)


    # This loop takes the first computer in one entry in leaves and connects it
    # to the last computer in the next entry in leaves (wrapping around at the
    # top).
    for i in range(0, len(leaves)):
	link_name = 'new-link%d' % i
	f = leaves[i][0]
	t = leaves[(i+1) % len(leaves)][-1]
	f.interface.append(topdl.Interface(substrate=[link_name]))
	t.interface.append(topdl.Interface(substrate=[link_name]))
	top.substrates.append(topdl.Substrate(name=link_name))

    # Now incorporate the new links
    top.incorporate_elements()
    return top

def output_ns(top, filename):
    '''
    Output ns2 with the OperatingSystem specifications converted to local OSIDs
    when known.
    '''
    def os_filter(e):
	'''
	Convert OS id to local equivalents:
	    FreeBSD -> FBSD8-STD
	    Lunix/Ubuntu -> Ubuntu-64-STD
	'''
	# Only work on Computers
	if not isinstance(e, topdl.Computer): return ''
	# Only handle exactly 1 specified OS
	if len(e.os) != 1 : return ''
	# String translation
	if e.os[0].name == 'FreeBSD': 
	    return 'tb-set-node-os ${%s} FBSD8-STD\n' % \
		    topdl.to_tcl_name(e.name)
	elif e.os[0].name == 'Linux' and e.os[0].distribution == 'Ubuntu':
	    return 'tb-set-node-os ${%s} Ubuntu1204-64-STD\n' % \
		    topdl.to_tcl_name(e.name)
	else:
	    return ''

    # Main line.  Open the file and write the otcl to it.
    try:
	f = open(filename, 'w')
	# Include the os_filter
	f.write(topdl.topology_to_ns2(top, 
	    filters=[os_filter], routing='Static'))
	f.close()
    except EnvironmentError, e:
	sys.exit('Cannot write %s: %s' % (e.filename, e.strerror))


# Parse get the file to parse and output file
if len(sys.argv) > 2:
    infile, outfile = (sys.argv[1], sys.argv[2])
else:
    sys.exit('Usage: %s infile outfile' % sys.argv[0])

top = topdl.topology_from_xml(filename=infile, top='experiment')
top = annotate_degree(top)
top = add_operating_system(top)
top = add_links(top)
output_ns(top, outfile)
